<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2011 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

class Storm_Test_ObjectWrapperException extends Exception {}

class Storm_Test_Mock extends StdClass {
  protected $_name;

  public function __construct($name) {
    $this->_name = $name;
  }

  public function toString() {
    return 'Storm_Test_Mock '.$this->_name;
  }


  public function __call($method, $args) {
    throw new Storm_Test_ObjectWrapperException('['.$this->toString().'] cannot execute "'.$method.'(' . implode(',', $this->_argsToString($args)) . ')"');
  }


  protected function _argsToString($args) {
    $new_args = [];
    foreach($args as $arg)
      $new_args [] = $this->_argToSring($arg);

      return $new_args;
  }


  protected function _argToSring($arg) {
    return is_object($arg)
      ? get_class($arg)
      : print_r($arg, true);
  }
}



class Storm_Test_ObjectWrapper {
  protected $_wrapped_object;
  protected $_call_trace = [];
  protected $_redirections = [];
  protected $_is_strict = false;


  /**
   * @param Object the oject to wrap
   * @return Storm_Test_ObjectWrapper
   */
  public static function on($object) {
    $wrapper = new self();
    return $wrapper->wrap($object);
  }


  /**
   * @return Storm_Test_ObjectWrapper
   */
  public static function mock($name = '') {
    if (!$name) {
      $stack = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
      $name = 'from ' . $stack[0]['file'] . ':' . $stack[0]['line'];
    }
    return self::on(new Storm_Test_Mock($name));
  }


  /**
   * @return Storm_Test_ObjectWrapper
   */
  public static function onLoaderOfModel($model_class) {
    $loader = call_user_func(array($model_class, 'getLoader'));
    $wrapper = self::on($loader);
    Storm_Model_Abstract::setLoaderFor($model_class, $wrapper);
    return $wrapper;
  }


  /**
   * Raises Exception if arguments don't match when searching redirections
   * @return Storm_Test_ObjectWrapper
   */
  public function beStrict() {
    $this->_is_strict = true;
    return $this;
  }


  /**
   * Compatibility method for fluent interface (before Storm_MethodRedirection::answer
   * did no return the wrapper
   * @return Storm_Test_ObjectWrapper
   */
  public function getWrapper() {
    return $this;
  }


  /**
   * @param Object the oject to wrap
   * @return Storm_Test_ObjectWrapper
   */
  public function wrap($object) {
    $this->_wrapped_object = $object;
    return $this;
  }


  /**
   * @return Object the wrapped object
   */
  public function getWrappedObject() {
    return $this->_wrapped_object;
  }


  protected function _raiseRedirectionNotFound($method, $args) {
    ob_start();
    var_dump($args);
    $dump = ob_get_contents();
    ob_end_clean();

    throw new Storm_Test_ObjectWrapperException(sprintf(
                                                        "Cannot find redirection for %s::%s(%s) %s",
                                                        get_class($this->_wrapped_object),
                                                        $method,
                                                        $dump,
                                                        ($this->_wrapped_object instanceof Storm_Test_Mock)
                                                        ? '[' . $this->_wrapped_object->toString() . ']'
                                                        : ''));
  }



  protected function _findRedirection($method, $args) {
    foreach($this->_redirections as $redirection) {
      if ($redirection->matchMethodAndArgs($method, $args))
          return $redirection;
    }

    if ($this->_is_strict)
      $this->_raiseRedirectionNotFound($method, $args);

    foreach($this->_redirections as $redirection) {
      if ($redirection->matchMethod($method) and !$redirection->expectArgs())
          return $redirection;
    }
  }


  public function __call($method, $args) {
    $this->_call_trace[$method] []= $args;

    if ($redirection = $this->_findRedirection($method, $args))
      return $redirection->getValueToAnswer($args);

    $result = call_user_func_array(array($this->_wrapped_object, $method),
                                   $args);

    if ($result === $this->_wrapped_object)
      return $this;
    return $result;
  }


  /**
   * @param $method_name string
   * @return boolean whether the method has been called or not
   */
  public function methodHasBeenCalled($method_name) {
    return array_key_exists($method_name, $this->_call_trace);
  }


  /**
   * @param $method_name string
   * @return boolean true if method has not been called
   */
  public function methodHasNotBeenCalled($method_name) {
    return !$this->methodHasBeenCalled($method_name);
  }


  /**
   * @param $method_name string
   * @param $params array
   * @return boolean true if method has been called with given params
   */
  public function methodHasBeenCalledWithParams($method_name, $params) {
    if (!$this->methodHasBeenCalled($method_name))
      return false;

    foreach ($this->_call_trace[$method_name] as $given_params) {
      if ($params == $given_params)
        return true;
    }
    return false;
  }

  /**
   * @param $method_name string
   * @return int the number of times this methos has been called
   */
  public function methodCallCount($method_name) {
    if (!$this->methodHasBeenCalled($method_name))
      return 0;
    return count($this->_call_trace[$method_name]);
  }


  /**
   * @param $method_name string
   * @return array the parameters given to the last call on this method
   */
  public function getAttributesForLastCallOn($method_name) {
    return $this->getAttributesForMethodCallAt($method_name, -1);
  }


  /**
   * @param $method_name string
   * @return mixed the first parameter given to the last call on this method
   */
  public function getFirstAttributeForLastCallOn($method_name) {
    return $this->getAttributesForLastCallOn($method_name)[0];
  }


  /**
   * @param $method_name string
   * @param $index int (note: -1 mean the last one, -2 the one before the last, ...)
   * @return mixed the first parameter given to the $indexth call on this method
   */
  public function getFirstAttributeForMethodCallAt($method_name, $index) {
    return $this->getAttributesForMethodCallAt($method_name, $index)[0];
  }


  /**
   * @param $method_name string
   * @param $index int (note: -1 mean the last one, -2 the one before the last, ...)
   * @return array the parameters given to the $indexth call on this method
   */
  public function getAttributesForMethodCallAt($method_name, $index) {
    if (!$this->methodHasBeenCalled($method_name))
      throw new Storm_Test_ObjectWrapperException("Method '$method_name' has never been called");

    $index = $index >=0 ? $index : count($this->_call_trace[$method_name]) + $index;
    return $this->_call_trace[$method_name][$index];
  }


  /**
   * @param string $method_name
   * @return Storm_Test_MethodRedirection
   */
  public function whenCalled($method_name) {
    $redirection = Storm_Test_MethodRedirection::onWrapper($this)->setMethod($method_name);
    array_unshift($this->_redirections, $redirection);
    return $redirection;
  }


  /**
   * @param string $method_name
   * @return Storm_Test_MethodRedirection
   */
  public function shouldNotBeCalled($method_name) {
    return $this->whenCalled($method_name)->never();
  }


  /**
   * Reset the mock
   * @return Storm_Test_ObjectWrapper
   */
  public function clearAllRedirections(){
    $this->_redirections = [];
    $this->_call_trace = [];
    return $this;
  }
}

?>