<?php
/*
  STORM is under the MIT License (MIT)

  Copyright (c) 2010-2011 Agence Française Informatique http://www.afi-sa.fr

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

*/

/**
 * Asserter for XPath matching with namespaces capabilities
 */
class Storm_Test_XPathXML {
  protected $_namespaces = array();

  public function registerNameSpace($prefix, $namespaceURI) {
    $this->_namespaces[$prefix] = $namespaceURI;
    return $this;
  }


  public function newXPathForXml($xml) {
    $document = $this->_getDocument($xml);
    $xpath = new DOMXPath($document);
    foreach ($this->_namespaces as $prefix => $uri) {
      if (!$xpath->registerNamespace($prefix, $uri))
        throw new Storm_Test_XPathException('Error registering namespace '.$prefix);
    }

    return $xpath;
  }


  public function assertXpathContentContains($xml, $path, $match, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);

    $nodeList = $this->_getNodeList($xml, $path);
    $this->_assertNodeListNotEmpty($nodeList, $path, $message);
    foreach ($nodeList as $node) {
      if ('' === $match && '' === $node->nodeValue)
        return;
      if (false !== strpos($node->nodeValue, $match))
        return;
    }

    throw new Storm_Test_XPathException($message . sprintf('Failed asserting node denoted by %s CONTAINS content "%s"',
                                                           $path, $match));
  }


  public function assertNotXpathContentContains($xml, $path, $match, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);

    $nodeList = $this->_getNodeList($xml, $path);
    $this->_assertNodeListNotEmpty($nodeList, $path, $message);
    foreach ($nodeList as $node) {
      if (false !== strpos($node->nodeValue, $match))
        throw new Storm_Test_XPathException($message . sprintf('Failed asserting node denoted by %s DOES NOT CONTAIN content "%s"',
                                                               $path, $match));
    }
  }


  public function assertXpath($xml, $path, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);

    $nodeList = $this->_getNodeList($xml, $path);
    $this->_assertNodeListNotEmpty($nodeList, $path, $message);
  }


  public function assertNotXpath($xml, $path, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);

    $nodeList = $this->_getNodeList($xml, $path);
    if ($nodeList->length > 0)
      throw new Storm_Test_XPathException($message . sprintf('Failed asserting that node denoted by "%s" DOES NOT EXIST',
                                                             $path));
  }


  public function assertXpathCount($xml, $path, $count, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);

    $nodeList = $this->_getNodeList($xml, $path);
    if ($nodeList->length != (int)$count)
      throw new Storm_Test_XPathException($message . sprintf('Failed asserting that node denoted by "%s" APPEARS EXACTLY %s TIMES',
                                                             $path, $count));
  }


  public function assertXmlVersion($xml, $version, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);
    $document = $this->_getDocument($xml);
    if ($document->xmlVersion != $version)
      throw new Storm_Test_XPathException($message . sprintf('Failed asserting that XML version of document "%s" EQUALS "%s"',
                                                             $document->xmlVersion, $version));
  }


  public function assertXmlEncoding($xml, $encoding, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);
    $document = $this->_getDocument($xml);
    if ($document->xmlEncoding != $encoding)
      throw new Storm_Test_XPathException($message . sprintf('Failed asserting that XML encoding of document "%s" EQUALS "%s"',
                                                             $document->xmlEncoding, $encoding));

  }


  protected function _getNodeList($xml, $path) {
    if (!$nodeList = $this->newXPathForXml($xml)->query($path))
      throw new Storm_Test_XPathException(sprintf('Error querying XPath "%s"', $path));
    return $nodeList;
  }


  protected function _assertNodeListNotEmpty($nodeList, $path, $message) {
    if ($nodeList->length == 0)
      throw new Storm_Test_XPathException($message . sprintf('Failed asserting that node denoted by "%s" EXISTS',
                                                             $path));
  }


  protected function _getDocument($xml) {
    $document = new DOMDocument();
    if (!$document->loadXML($xml))
      throw new Storm_Test_XPathException('Error parsing document');
    return $document;
  }
}

?>