<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2011 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

class Storm_Test_LoaderTest extends Storm_Test_ModelTestCase {
  protected $_loader;
  protected $_table;

  public function setUp() {
    parent::setUp();
    $this->_loader = Storm_Test_Mock_User::getLoader();
    $this->_table = Storm_Test_ObjectWrapper::mock();

    $this->_loader->setTable($this->_table);
  }


  /** @test */
  public function findAllShouldAcceptASQLQuery() {
    $this->_table
        ->whenCalled('getAdapter')->answers($this->_table)
        ->whenCalled('fetchAll')->answers(array());

    $this->assertEquals(array(), $this->_loader->findAll('SELECT * FROM USERS'));
    $this->assertEquals('SELECT * FROM USERS', $this->_table->getFirstAttributeForLastCallOn('fetchAll'));
  }


  /** @test */
  public function countByWithWhereShouldBuildRightSQL() {
    $select = Storm_Test_ObjectWrapper::mock()
      ->whenCalled('from')
      ->with($this->_table, ['count(*) as numberof'])
      ->answers(null)

      ->whenCalled('where')
      ->with('nom like "%zork%"')
      ->answers(null)

      ->beStrict();

    $this->_table
      ->whenCalled('select')->answers($select)

      ->whenCalled('fetchAll')
      ->with($select)
      ->answers(new Zend_Db_Table_Rowset(array('data' => array(array('numberof' => 3)))));


    $this->assertEquals(3, $this->_loader->countBy(array('where' => 'nom like "%zork%"')));
  }


  /** @test */
  public function withLimitPageFindAllByShouldBuildSql() {
    $this->_table
      ->whenCalled('select')->answers($select = Storm_Test_ObjectWrapper::mock())
      ->whenCalled('fetchAll')->with($select)
      ->answers(new Zend_Db_Table_Rowset(array()));

    $select
      ->whenCalled('where')->answers($select)
      ->whenCalled('limitPage')->answers($select);

    $this->_loader->findAllBy(array('limitPage' => array(2, 80)));

    $this->assertEquals(array(2, 80), $select->getAttributesForLastCallOn('limitPage'));
  }



  /** @test */
  public function withNullValueFindAllByShouldBuildSqlWithIsNullStatement() {
    $select = Storm_Test_ObjectWrapper::mock()
      ->whenCalled('where')
      ->with('parent_id is null')
      ->answers(null)

      ->whenCalled('where')
      ->with('cat_id=?', 3)
      ->answers(null)

      ->beStrict();

    $this->_table
      ->whenCalled('select')->answers($select)

      ->whenCalled('fetchAll')
      ->with($select)
      ->answers([]);

    $this->assertSame([],
                      $this->_loader->findAllBy(['parent_id' => null,
                                                 'scope' => ['cat_id' => 3]]));
  }



  /** @test */
  public function whenAnInstanceIsAlreadyLoadedFindAllShouldUseItInsteadOfCreatingANewInstance() {
    $existing_user = Storm_Test_Mock_User::newInstanceWithId(34, ['name' => 'jc']);

    $this->_table
      ->whenCalled('fetchAll')
      ->with(null)
      ->answers(new Zend_Db_Table_Rowset(['data' => [
                                           ['id' => 29,
                                            'name' => 'lp'],

                                           ['id' => 34,
                                            'name' => 'jc'] ]
                                           ]));

    $this->assertSame($existing_user, $this->_loader->findAll()[1]);
  }


  /** @test */
  public function withNegatedValueShouldBeAsExpected() {
    $this->_table
      ->whenCalled('select')->answers($select = Storm_Test_ObjectWrapper::mock())
      ->whenCalled('fetchAll')->with($select)
      ->answers(new Zend_Db_Table_Rowset([]));

    $select
      ->whenCalled('where')->answers($select);

    $this->_loader->findAllBy(['name not' => 'Harlock']);
    $this->assertEquals(['name!=?', 'Harlock'], $select->getAttributesForLastCallOn('where'));

    $this->_loader->findAllBy(['name not' => ['Harlock', 'Nausicaa']]);
    $this->assertEquals(['name not in (?)', ['Harlock', 'Nausicaa']],
                        $select->getAttributesForLastCallOn('where'));

    $this->_loader->findAllBy(['name not' => null]);
    $this->assertEquals(['name is not null'],
                        $select->getAttributesForLastCallOn('where'));
  }
}